@@ -1,4 +1,5 @@
 from .interaction import *
-from .slash_interaction import *
+from .app_command_interaction import *
 from .message_interaction import *
 from .message_components import *
+from .application_command import *
